/* -*- mode: C++; c-file-style: "gnu" -*- */
/* object.h: D-BUS / Qt object base class
 *
 * Copyright (C) 2005  James Thorniley <james@uncommonlygood.co.uk>
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef DBUS_QT_OBJECT_H
#define DBUS_QT_OBJECT_H

#include <dbus/dbus.h>

class QString;

namespace DBusQt
{

  class Message;
  class Connection;
  
  /**
  @brief An object capable of receiving D-BUS method invocations and emitting D-BUS signals.
  
  <p>Hopefully, in the future, subclasses will be created from some kind of IDL compiler.
  For now, the way to implement an object is to inherit from this class, working out some
  way to provide a path and name to identify this object on the bus. Register this object
  on a bus using Connection::registerObject(). Reimplement process()
  to receive object messages.
  */
  class Object
  {
    public:
    
      /**
      Create a new Object.
      
      @todo Automatically attach to a D-BUS connection if possible
      */
      Object( );
    
      ~Object();
      
      /**
      Receives incoming messages. Subclasses should reimplement this
      to invoke object methods and send a reply if
      appropriate.
      
      @param message The D-BUS message that has been received.
      @return A DBusHandlerResult. This should be one of 
      @arg @c DBUS_HANDLER_RESULT_HANDLED If this method requested was invoked;
      @arg @c DBUS_HANDLER_RESULT_NOT_YET_HANDLED If the method requested
      could not be invoked; or
      @arg @c DBUS_HANDLER_RESULT_NEED_MEMORY If the method could not be
      completed because of a lack of memory.
      */
      virtual DBusHandlerResult process( Connection &connection, const Message &message );
      
    private:
      class Private;
      Private *d;
    
  };

}

#endif
